package org.processmining.plugins.bpmnminer.causalnet;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.util.HashSet;
import java.util.Set;

import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.shapes.Decorated;
import org.processmining.models.shapes.Ellipse;
import org.processmining.models.shapes.Rectangle;

public class CausalGraphNode 
	extends AbstractDirectedGraphNode 
	implements Decorated {

	private final int code;
	private final boolean andNode;
	private final Set<Integer> incoming;
	private final Set<Integer> outgoing;
	
	private final CausalGraph causalGraph;
	
	public CausalGraphNode(CausalGraph g, int c, Set<Integer> i, Set<Integer> o) {
		super();
		causalGraph = g;
		code = c;
		incoming = i;
		outgoing = o;
		andNode = true;
		
		getAttributeMap().put(AttributeMap.SHAPE, new Ellipse());
		getAttributeMap().put(AttributeMap.LABEL, "+");
		getAttributeMap().put(AttributeMap.SHOWLABEL, true);
		getAttributeMap().put(AttributeMap.RESIZABLE, false);
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(20, 20));
		getAttributeMap().put(AttributeMap.FILLCOLOR, Color.GRAY);
		getAttributeMap().put(AttributeMap.SHOWLABEL, true);
	}
	
	public CausalGraphNode(CausalGraph g, int c, String label) {
		super();
		causalGraph = g;
		code = c;
		incoming = new HashSet<Integer>();
		outgoing = new HashSet<Integer>();
		andNode = false;
		
		getAttributeMap().put(AttributeMap.SHAPE, new Rectangle());
		getAttributeMap().put(AttributeMap.LABEL, label);
		getAttributeMap().put(AttributeMap.SHOWLABEL, true);
		getAttributeMap().put(AttributeMap.RESIZABLE, false);
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(120, 70));
		getAttributeMap().put(AttributeMap.FILLCOLOR, Color.WHITE);
		getAttributeMap().put(AttributeMap.SHOWLABEL, true);
	}

	public void setLabel(String newLabel) {
		getAttributeMap().remove(AttributeMap.LABEL);
		getAttributeMap().put(AttributeMap.LABEL, newLabel);
	}

	public void decorate(Graphics2D g2d, double x, double y, double width, double height) {
		
	}

	public AbstractDirectedGraph<?, ?> getGraph() {
		return causalGraph;
	}

	public int getCode() {
		return code;
	}
	
	public boolean isAnd() {
		return andNode;
	}
	
	public Set<Integer> getIncoming() {
		return incoming;
	}
	
	public Set<Integer> getOutgoing() {
		return outgoing;
	}
}
