package org.processmining.plugins.bpmnminer.converter;

import java.util.Set;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.dependencygraph.DependencyNet;

@Plugin(name = "Convert Causal Net to Dependency Net", 
		parameterLabels = { "Causal Net" }, 
		returnLabels = {"Dependency Net" }, 
		returnTypes = { DependencyNet.class }, 
		userAccessible = true, 
		help = "Converts Causal Net into Causal Graph")

public class CausalNetToDependencyNet {
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, 
			author = "Seppe K.L.M. vanden Broucke", 
			email = "seppe.vandenbroucke@econ.kuleuven.be", 
			website = "http://econ.kuleuven.be")
	@PluginVariant(variantLabel = "Default Settings", requiredParameterLabels = { 0 })
	
	public static DependencyNet convert(PluginContext context, CausalNet cn) {
		DependencyNet returns = toDependencyNet(cn);
		return returns;
	}

	public static DependencyNet toDependencyNet(CausalNet causalNet) {
		DependencyNet graph = new DependencyNet(causalNet.getLog());
		for (int t : causalNet.getTasks()) {
			graph.addTask(t);
			graph.setLabel(t, causalNet.getLabel(t));
		}
		for (int t : causalNet.getTasks()) {
			Set<Integer> inputs = causalNet.getInputTasks(t);
			Set<Integer> outputs = causalNet.getOutputTasks(t);
			for (int i : inputs)
				graph.setArc(i, t);
			for (int o : outputs)
				graph.setArc(t, o);
			if (inputs.size() == 0)
				graph.setStartTask(t);
			if (outputs.size() == 0)
				graph.setEndTask(t);
		}
		return graph;
	}

}