package org.processmining.plugins.bpmnminer.converter;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.flexiblemodel.FlexFactory;
import org.processmining.models.flexiblemodel.FlexNode;
import org.processmining.models.flexiblemodel.SetFlex;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.types.EventLogTaskMapper;

@Plugin(name = "Convert Causal Net to Flex Net", 
		parameterLabels = { "Causal Net" }, 
		returnLabels = { "Flex Net" }, 
		returnTypes = { Flex.class }, 
		userAccessible = true, 
		help = "Converts Causal Net into Flex Net")

public class CausalNetToFlex {
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, 
			author = "Seppe K.L.M. vanden Broucke", 
			email = "seppe.vandenbroucke@econ.kuleuven.be", 
			website = "http://econ.kuleuven.be")
	@PluginVariant(variantLabel = "Default Settings", requiredParameterLabels = { 0 })
	
	public static Flex convert(PluginContext context, CausalNet f) {
		Flex returns = toFlexNet(f);
		return returns;
	}

	public static Flex toFlexNet(CausalNet f) {
		final Flex flexDiagram = FlexFactory.newFlex("Converted Flex Model");
		Map<Integer,FlexNode> nodes = new HashMap<Integer,FlexNode>();
		
		for (int node : f.getTasks()) {
			nodes.put(node, flexDiagram.addNode(f.getLabel(node)));
		}
		
		for (int node : f.getTasks()) {
			for (int out : f.getOutputTasks(node)) {
				flexDiagram.addArc(nodes.get(node), nodes.get(out));
			}
			
			if(f.getInputTasks(node).size() > 0){
				for (Set<Integer> set : f.getInputSets(node)){
					SetFlex temp = new SetFlex();
					for(int i : set){	
						temp.add(nodes.get(i));
					}
					nodes.get(node).addInputNodes(temp);
				}
			}
			else nodes.get(node).addInputNodes(new SetFlex());

			if(f.getOutputTasks(node).size() > 0){
				for (Set<Integer> set : f.getOutputSets(node)){
					SetFlex temp = new SetFlex();
					for(int i : set){	
						temp.add(nodes.get(i));
					}
					nodes.get(node).addOutputNodes(temp);
				}
			}
			else nodes.get(node).addOutputNodes(new SetFlex());
		}
		
		return flexDiagram;
	}
	
	public static CausalNet toCausalNet(Flex f, EventLogTaskMapper m) {
		CausalNet causalNet = new CausalNet();
		
		for (FlexNode n : f.getNodes()) {
			int nCode = m.getTaskMapping(n.getLabel(), new String[]{}, new String[]{});
			causalNet.addTask(nCode);
			for (SetFlex in : n.getInputNodes()) {
				Set<Integer> set = new HashSet<Integer>();
				for (FlexNode i : in)
					set.add(m.getTaskMapping(i.getLabel(), new String[]{}, new String[]{}));
				if (set.size() > 0)
					causalNet.addInputSet(nCode, set);
			}
			
			for (SetFlex out : n.getOutputNodes()) {
				Set<Integer> set = new HashSet<Integer>();
				for (FlexNode o : out)
					set.add(m.getTaskMapping(o.getLabel(), new String[]{}, new String[]{}));
				if (set.size() > 0)
					causalNet.addOutputSet(nCode, set);
			}
		}

		return causalNet;
	}
	

}