package org.processmining.plugins.bpmnminer.converter;

import java.util.HashSet;
import java.util.Set;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.dependencygraph.DependencyNet;

@Plugin(name = "Convert Dependency Net to Fuzzy Causal Net", 
		parameterLabels = { "Dependency Net" }, 
		returnLabels = {"Causal Net" }, 
		returnTypes = { CausalNet.class }, 
		userAccessible = true, 
		help = "Converts Dependency Net into Fuzzy Causal Net (AND splits and XOR joins)")

public class DependencyNetToCausalNet {
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, 
			author = "Seppe K.L.M. vanden Broucke", 
			email = "seppe.vandenbroucke@econ.kuleuven.be", 
			website = "http://econ.kuleuven.be")
	@PluginVariant(variantLabel = "Default Settings", requiredParameterLabels = { 0 })
	
	public static CausalNet convert(PluginContext context, DependencyNet dn) {
		CausalNet returns = toCausalNet(dn);
		return returns;
	}

	public static CausalNet toCausalNet(DependencyNet dependencyNet) {
		CausalNet causalNet = new CausalNet();
		for (int t : dependencyNet.getTasks()) {
			causalNet.addTask(t);
			causalNet.addOutputSet(t, dependencyNet.getTaskOutputs(t));
			for (int i : dependencyNet.getTaskInputs(t)) {
				Set<Integer> set = new HashSet<Integer>();
				set.add(i);
				causalNet.addInputSet(t, set);
			}
		}
		return causalNet;
	}
	

}