package org.processmining.plugins.bpmnminer.converter;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.flexiblemodel.Flex;
import org.processmining.models.flexiblemodel.FlexNode;
import org.processmining.models.flexiblemodel.SetFlex;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.types.EventLogTaskMapper;

@Plugin(name = "Convert Flex Net to Causal Net", 
		parameterLabels = { "Flex Net" }, 
		returnLabels = {"Causal Net" }, 
		returnTypes = { CausalNet.class }, 
		userAccessible = true, 
		help = "Converts Flex Net into Causal Net")

public class FlexToCausalNet {
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, 
			author = "Seppe K.L.M. vanden Broucke", 
			email = "seppe.vandenbroucke@econ.kuleuven.be", 
			website = "http://econ.kuleuven.be")
	@PluginVariant(variantLabel = "Default Settings", requiredParameterLabels = { 0 })
	
	public static CausalNet convert(PluginContext context, Flex f) {
		CausalNet returns = toCausalNet(f);
		return returns;
	}

	public static CausalNet toCausalNet(Flex f) {
		Map<FlexNode, Integer> map = new HashMap<FlexNode, Integer>();
		int t = 1;
		for (FlexNode n : f.getNodes())
			map.put(n, ++t);
		
		CausalNet causalNet = new CausalNet();
		
		for (FlexNode n : f.getNodes()) {
			causalNet.addTask(map.get(n));
			causalNet.setLabel(map.get(n), n.getLabel().trim()+"+complete");
			for (SetFlex in : n.getInputNodes()) {
				Set<Integer> set = new HashSet<Integer>();
				for (FlexNode i : in)
					set.add(map.get(i));
				if (set.size() > 0)
					causalNet.addInputSet(map.get(n), set);
			}
			
			for (SetFlex out : n.getOutputNodes()) {
				Set<Integer> set = new HashSet<Integer>();
				for (FlexNode o : out)
					set.add(map.get(o));
				if (set.size() > 0)
					causalNet.addOutputSet(map.get(n), set);
			}
		}

		return causalNet;
	}
	
	public static CausalNet toCausalNet(Flex f, EventLogTaskMapper m) {
		CausalNet causalNet = new CausalNet();
		
		for (FlexNode n : f.getNodes()) {
			int nCode = m.getTaskMapping(n.getLabel(), new String[]{}, new String[]{});
			causalNet.addTask(nCode);
			for (SetFlex in : n.getInputNodes()) {
				Set<Integer> set = new HashSet<Integer>();
				for (FlexNode i : in)
					set.add(m.getTaskMapping(i.getLabel(), new String[]{}, new String[]{}));
				if (set.size() > 0)
					causalNet.addInputSet(nCode, set);
			}
			
			for (SetFlex out : n.getOutputNodes()) {
				Set<Integer> set = new HashSet<Integer>();
				for (FlexNode o : out)
					set.add(m.getTaskMapping(o.getLabel(), new String[]{}, new String[]{}));
				if (set.size() > 0)
					causalNet.addOutputSet(nCode, set);
			}
		}

		return causalNet;
	}
	

}