package org.processmining.plugins.bpmnminer.fitness;

import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.converter.CausalNetToDependencyNet;
import org.processmining.plugins.bpmnminer.dependencygraph.DependencyNet;
import org.processmining.plugins.bpmnminer.types.EventLogTaskMapper;

public class FlowFitness extends BasicFitness {

	protected int badFlows;
	protected int goodFlows;
	
	protected int badStarts;
	protected int goodStarts;
	
	protected int badEnds;
	protected int goodEnds;
	
	protected int badTraces;
	protected int goodTraces;
	
	protected DependencyNet dependencyNet;
	
	public FlowFitness(EventLogTaskMapper m, CausalNet n) {
		super(m, n);
		dependencyNet = CausalNetToDependencyNet.toDependencyNet(n);
	}
	
	public void reset() {
		badFlows = 0;
		goodFlows = 0;
		badStarts = 0;
		goodStarts = 0;
		badEnds = 0;
		goodEnds = 0;
		badTraces = 0;
		goodTraces = 0;
	}
	
	public double getFitness() {
		return (double) goodFlows / ((double)goodFlows + (double)badFlows);
	}
	
	public void replayTrace(XTrace trace, int traceCount) {
		boolean seqBadStart = false;
		boolean seqBadEnd = false;
		int seqBadFlows = 0;
		int seqGoodFlows = 0;
		
		String[] eventrow = mapper.classifyTrace(trace);
		int[] taskrow = mapper.getIntegerTrace(trace);
		
		for (int i = 0; i < taskrow.length-1; i++) {
			Integer currentTask = getBestTask(eventrow, taskrow, i, null);
			Integer nextTask = getBestTask(eventrow, taskrow, i+1, null);
			taskrow[i] = (currentTask == null) ? -9999 : currentTask;
				
			if (currentTask == null || nextTask == null || !dependencyNet.isArc(currentTask, nextTask))
				seqBadFlows++;
			else
				seqGoodFlows++;
		}
		
		if (dependencyNet.getStartTask() != taskrow[0])
			seqBadStart = true;
		if (dependencyNet.getEndTask() != taskrow[taskrow.length-1])
			seqBadEnd = true;
		
		badFlows += seqBadFlows * traceCount;
		goodFlows += seqGoodFlows * traceCount;
		
		if (seqBadStart)
			badStarts += traceCount;
		else
			goodStarts += traceCount;
		
		if (seqBadEnd)
			badEnds += traceCount;
		else
			goodEnds += traceCount;
		
		if (seqBadStart || seqBadEnd || seqBadFlows > 0)
			badTraces += traceCount;
		else
			goodTraces += traceCount;
	}

}
