package org.processmining.plugins.bpmnminer.importexport;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Set;

import org.deckfour.spex.SXDocument;
import org.deckfour.spex.SXTag;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;

@Plugin(name = "Export Causal Net",
	returnLabels = {}, 
	returnTypes = {}, 
	parameterLabels = { "CausalNet", "File" }, 
	userAccessible = true)
@UIExportPlugin(description = "CNET files", extension = "cnet")
public class ExportCNet {

	@PluginVariant(variantLabel = "CNET export", requiredParameterLabels = { 0, 1 })
	public void exportCausalNetToCNETFile(UIPluginContext context, CausalNet net, File file) throws IOException {
		SXDocument doc = new SXDocument(file);
		SXTag root = doc.addNode("causalnet");
		root.addAttribute("size", ""+net.getTasks().size());
		
		SXTag activities = root.addChildNode("activities");
		for (int t : net.getTasks()) {
			SXTag activity = activities.addChildNode("activity");
			activity.addAttribute("identifier", t+"");
			SXTag activityCla = activity.addChildNode("label");
			activityCla.addTextNode(net.getLabel(t));
		}
		
		SXTag connections = root.addChildNode("connections");
		for (int t : net.getTasks()) {
			SXTag activity = connections.addChildNode("activity");
			activity.addAttribute("identifier", t+"");
			List<Set<Integer>> inputSet = net.getInputSets(t);
			List<Set<Integer>> outputSet = net.getOutputSets(t);
			
			SXTag inputsets = activity.addChildNode("inputsets");
			for (int j = 0; j < inputSet.size(); j++) {
				Set<Integer> subset = inputSet.get(j);
				SXTag subsettag = inputsets.addChildNode("inputset");
				for (int k : subset) {
					SXTag subsetact = subsettag.addChildNode("activity");
					subsetact.addAttribute("identifier", k+"");
				}
			}
			
			SXTag outputsets = activity.addChildNode("outputsets");
			for (int j = 0; j < outputSet.size(); j++) {
				Set<Integer> subset = outputSet.get(j);
				SXTag subsettag = outputsets.addChildNode("outputset");
				for (int k : subset) {
					SXTag subsetact = subsettag.addChildNode("activity");
					subsetact.addAttribute("identifier", k+"");
				}
			}
		}
			
		doc.close();
		
	}

}
