package org.processmining.plugins.bpmnminer.plugins;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.bpmn.BPMNDiagram;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.types.EventLogTaskMapper;
import org.processmining.plugins.bpmnminer.types.MinerSettings;
import org.processmining.plugins.bpmnminer.util.CausalNetBPMNDiagramConnection;

@Plugin(name = "BPMN Checker", 
		parameterLabels = {"BPMNDiagram", "Log"},
		returnLabels = {"BPMN Diagram"},
		returnTypes = {
			BPMNDiagram.class
		},
		userAccessible = true,
		help = "Check conformance on BPMN Miner mined BPMN model")

public class BPMNCheckerPlugin {
	
	@UITopiaVariant(uiLabel = "Conformance analysis on BPMN model",
			affiliation = UITopiaVariant.EHV,
			author = "Seppe K.L.M. vanden Broucke",
			email = "seppe.vandenbroucke@econ.kuleuven.be",
			website = "http://econ.kuleuven.be")
	
	@PluginVariant(variantLabel = "Wizard settings", requiredParameterLabels = { 0, 1 })
	public static BPMNDiagram conformanceCheckBPMN(UIPluginContext context, BPMNDiagram originalDiagram, XLog log) {
		MinerSettings settings = new MinerSettings();
		double nom = log.size() / ((double)log.size() + (double)settings.dependencyDivisor);
		if (nom <= 0D) nom = 0D;
		if (nom >= 0.9D) nom = 0.9D;
		settings.dependencyThreshold = nom;
		settings.l1lThreshold = nom;
		settings.l2lThreshold = nom;
		settings.suppressFitnessReport = false;
		EventLogTaskMapper mapper = new EventLogTaskMapper(log, settings.classifier);
		mapper.setup(settings.backwardContextSize, 
				settings.forwardContextSize, 
				settings.useUniqueStartEndTasks, 
				settings.collapseL1l,
				settings.duplicateThreshold);
		
		CausalNet causalNet;
		try {
			CausalNetBPMNDiagramConnection causalNetConnection = context.getConnectionManager().getFirstConnection(
					CausalNetBPMNDiagramConnection.class, context, originalDiagram);
			causalNet = causalNetConnection.getCausalNet();
		} catch (ConnectionCannotBeObtained e) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		BPMNDiagram diagram = BPMNMinerPlugin.calculateFitness(originalDiagram, mapper, causalNet, log);
				
		return diagram;
	}
	
}
