package org.processmining.plugins.bpmnminer.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.fitness.AbstractFitness;
import org.processmining.plugins.bpmnminer.fitness.FlowFitness;
import org.processmining.plugins.bpmnminer.fitness.FuzzyFitness;
import org.processmining.plugins.bpmnminer.fitness.ICSFitness;
import org.processmining.plugins.bpmnminer.fitness.PMFitness;
import org.processmining.plugins.bpmnminer.fitness.RecallFitness;
import org.processmining.plugins.bpmnminer.miner.FodinaMiner;
import org.processmining.plugins.bpmnminer.types.EventLogTaskMapper;
import org.processmining.plugins.bpmnminer.types.IntegerEventLog;
import org.processmining.plugins.bpmnminer.types.MinerSettings;
import org.processmining.plugins.bpmnminer.types.RelationalMetrics;
import org.processmining.plugins.bpmnminer.ui.AbstractParameterPanel;
import org.processmining.plugins.bpmnminer.ui.FullParameterPanel;
import org.processmining.plugins.bpmnminer.ui.SimpleParameterPanel;
import org.processmining.plugins.kutoolbox.groupedlog.GroupedXLog;

@Plugin(name = "Fodina Miner", 
		parameterLabels = {"Log", "Settings"},
		returnLabels = {"Causal Net", "Miner Settings"},
		returnTypes = {
			CausalNet.class,
			MinerSettings.class
		},
		userAccessible = true,
		help = "Causal Net Miner Based on Heuristics")

public class FodinaMinerPlugin {
	
	@UITopiaVariant(uiLabel = "Mine Causal net with Fodina",
			affiliation = UITopiaVariant.EHV,
			author = "Seppe K.L.M. vanden Broucke",
			email = "seppe.vandenbroucke@econ.kuleuven.be",
			website = "http://econ.kuleuven.be")
	
	@PluginVariant(variantLabel = "Wizard settings", requiredParameterLabels = { 0 })
	public static Object[] ExecutePlugin(UIPluginContext context, XLog log) {
		MinerSettings settings = new MinerSettings();
		double nom = log.size() / ((double)log.size() + (double)settings.dependencyDivisor);
		if (nom <= 0D) nom = 0D;
		if (nom >= 0.9D) nom = 0.9D;
		settings.dependencyThreshold = nom;
		settings.l1lThreshold = nom;
		settings.l2lThreshold = nom;
		settings.suppressFitnessReport = false;
		
		AbstractParameterPanel parameters = new FullParameterPanel(settings);
		InteractionResult result = context.showConfiguration("Miner Parameters", parameters);
		
		if (result.equals(InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		return runMiner(context, log, parameters.getSettings());
	}
	
	/*
	@UITopiaVariant(uiLabel = "Mine Causal net with Fodina (Single Trace Test)",
			affiliation = UITopiaVariant.EHV,
			author = "Seppe K.L.M. vanden Broucke",
			email = "seppe.vandenbroucke@econ.kuleuven.be",
			website = "http://econ.kuleuven.be")
	
	@PluginVariant(variantLabel = "Wizard settings", requiredParameterLabels = { 0 })
	*/
	public static Object[] TestSingleTracePlugin(UIPluginContext context, XLog log) {
		MinerSettings settings = new MinerSettings();
		double nom = log.size() / ((double)log.size() + (double)settings.dependencyDivisor);
		if (nom <= 0D) nom = 0D;
		if (nom >= 0.9D) nom = 0.9D;
		settings.dependencyThreshold = nom;
		settings.l1lThreshold = nom;
		settings.l2lThreshold = nom;
		
		AbstractParameterPanel parameters = new FullParameterPanel(settings);
		InteractionResult result = context.showConfiguration("Miner Parameters", parameters);
		
		if (result.equals(InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		settings = parameters.getSettings();
		
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		double fitting = 0;
		for (XTrace trace : log) {
			XLog copy = factory.createLog();
			copy.add(trace);

			Object[] r = runMiner(context, copy, settings);
			EventLogTaskMapper mapper = new EventLogTaskMapper(copy, settings.classifier);
			mapper.setup(settings.backwardContextSize, 
					settings.forwardContextSize, 
					settings.useUniqueStartEndTasks, 
					settings.collapseL1l,
					settings.duplicateThreshold);
			EventLogTaskMapper fitnessMapper = EventLogTaskMapper.createMapping((CausalNet) r[0], settings.classifier);
			
			AbstractFitness fitness = new ICSFitness(fitnessMapper, (CausalNet) r[0]);
			GroupedXLog gl = new GroupedXLog(copy);
			fitness.replayLog(gl);
			if (fitness.getFitness() >= 1d)
				fitting++;
			else {
				System.out.println("Non fitting trace: "+gl.traceHash(trace));
			}
		}
		
		System.out.println("Fitting percentage: "+(fitting / (double) log.size()));
		
		context.getFutureResult(0).cancel(true);
		return null;
	}
	
	@UITopiaVariant(uiLabel = "Mine Causal net with Fodina (Simple Setup)",
			affiliation = UITopiaVariant.EHV,
			author = "Seppe K.L.M. vanden Broucke",
			email = "seppe.vandenbroucke@econ.kuleuven.be",
			website = "http://econ.kuleuven.be")
	
	@PluginVariant(variantLabel = "Wizard settings", requiredParameterLabels = { 0 })
	public static Object[] ExecuteSimplePlugin(UIPluginContext context, XLog log) {
		MinerSettings settings = new MinerSettings();
		double nom = log.size() / ((double)log.size() + (double)settings.dependencyDivisor);
		if (nom <= 0D) nom = 0D;
		if (nom >= 0.9D) nom = 0.9D;
		settings.dependencyThreshold = nom;
		settings.l1lThreshold = nom;
		settings.l2lThreshold = nom;
		settings.suppressFitnessReport = false;
		
		AbstractParameterPanel parameters = new SimpleParameterPanel(settings);
		InteractionResult result = context.showConfiguration("Miner Parameters", parameters);
		
		if (result.equals(InteractionResult.CANCEL)) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		
		return runMiner(context, log, parameters.getSettings());
	}
	
	@PluginVariant(variantLabel = "Default settings", requiredParameterLabels = { 0 })
	public static Object[] ExecutePlugin(PluginContext context, XLog log) {
		return runMiner(context, log, new MinerSettings());
	}

	@PluginVariant(variantLabel = "Given settings", requiredParameterLabels = { 0, 1 })
	public static Object[] runMiner(PluginContext context, XLog log, MinerSettings settings) {
		fireInitializing(context);
		
		// Build integer event log
		fireMapping(context);
		EventLogTaskMapper mapper = new EventLogTaskMapper(log, settings.classifier);
		mapper.setup(settings.backwardContextSize, 
				settings.forwardContextSize, 
				settings.useUniqueStartEndTasks, 
				settings.collapseL1l,
				settings.duplicateThreshold);
		IntegerEventLog ieLog = mapper.getIntegerLog();
		
		// Mine relational metrics
		fireMetricConstruction(context);
		RelationalMetrics rMiner = new RelationalMetrics(ieLog, settings);
		
		// Mine causal net
		fireMining(context);
		FodinaMiner miner = new FodinaMiner(ieLog, rMiner, settings);
		CausalNet net = miner.mine();
		
		if (!settings.suppressFitnessReport) {
			showFitness(mapper, net);
		}
			
		return new Object[] {net, settings};
	}
	
	public static void showFitness(EventLogTaskMapper mapper, CausalNet net) {
		System.out.println("-------------- FITNESS --------------");
		System.out.println(mapper.toString());
		
		AbstractFitness fitness1 = new ICSFitness(mapper, net);
		fitness1.replayLog(mapper.getGroupedLog());
		System.out.println("ICS FITNESS: "+fitness1.getFitness());
		
		AbstractFitness fitness2 = new RecallFitness(mapper, net);
		fitness2.replayLog(mapper.getGroupedLog());
		System.out.println("RECALL: "+fitness2.getFitness());
		
		AbstractFitness fitness3 = new PMFitness(mapper, net);
		fitness3.replayLog(mapper.getGroupedLog());
		System.out.println("PM: "+fitness3.getFitness());

		AbstractFitness fitness4 = new FlowFitness(mapper, net);
		fitness4.replayLog(mapper.getGroupedLog());
		System.out.println("Flow: "+fitness4.getFitness());

		AbstractFitness fitness5 = new FuzzyFitness(mapper, net);
		fitness5.replayLog(mapper.getGroupedLog());
		System.out.println("Fuzzy: "+fitness5.getFitness());
	}
	
	public static void showFitness(XLog log, CausalNet net) {
		EventLogTaskMapper mapper = EventLogTaskMapper.createMapping(net, log, XLogInfoImpl.STANDARD_CLASSIFIER);
		showFitness(mapper, net);
	}
	
	private static void fireInitializing(PluginContext context) {
		if (context != null) {
			context.getProgress().setMinimum(0);
			context.getProgress().setMaximum(5);
			context.getProgress().setValue(1);
			context.log("Starting Fodina miner...");
		}
	}
	
	private static void fireMapping(PluginContext context) {
		if (context != null) {
			context.getProgress().setValue(2);
			context.log("Constructing tasklog; mining and filtering duplicates...");
			context.getProgress().setIndeterminate(true);
		}
	}
	
	private static void fireMetricConstruction(PluginContext context) {
		if (context != null) {
			context.getProgress().setIndeterminate(false);
			context.getProgress().setMinimum(0);
			context.getProgress().setMaximum(5);
			context.getProgress().setValue(3);
			context.log("Constructing relational metrics...");
		}
	}
	
	private static void fireMining(PluginContext context) {
		if (context != null) {
			context.getProgress().setValue(4);
			context.log("Mining causal net...");
		}
	}
}
