package org.processmining.plugins.bpmnminer.ui;

import javax.swing.JCheckBox;

import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.processmining.plugins.bpmnminer.types.MinerSettings;
import org.processmining.plugins.kutoolbox.ui.FancyIntegerSlider;

public class BPMNParameterPanel extends AbstractParameterPanel {
	private static final long serialVersionUID = -1586415938421148771L;

	private FancyIntegerSlider tD, tPG;
	private JCheckBox cUSE, cAL2, cPL2;
	private JCheckBox reportFitness, mineSwimLanes, filterLog;
	
	public BPMNParameterPanel(){
		this(new MinerSettings());
	}
	
	public BPMNParameterPanel(MinerSettings settings){
		super(settings);
	}
	
	protected void init(){
		this.addHeading("Options", 1);

		this.addLabel("Dependency threshold:", 2);
		tD = this.addIntegerSlider(0, (int) (settings.dependencyThreshold * 100D), 100, 2);
		
		this.addLabel("Precision / generalisation threshold (higher = stronger):", 3);
		tPG = this.addIntegerSlider(-100, (int) (settings.patternThreshold * 100D), 100, 3);
		
		cUSE = this.addCheckbox("Use artificial start/end activities", settings.useUniqueStartEndTasks, 5, false);
		cAL2 = this.addCheckbox("Resolve binary conflicts where possible", settings.preferAndToL2l, 6, false);
		cPL2 = this.addCheckbox("Prevent length two loops between repeating activities", settings.preventL2lWithL1l, 7, false);
		
		mineSwimLanes = this.addCheckbox("Mine BPMN swimlanes", true, 9, true);
		reportFitness = this.addCheckbox("Report fitness conformance", true, 10, true);
		filterLog = this.addCheckbox("Filter log before mining", false, 11, true);
		
		this.updateFields();		
	}
	
	public void applySettings(MinerSettings settings){
		this.tD.setValue((int) (settings.dependencyThreshold * 100D));
		this.tPG.setValue((int) (settings.patternThreshold * 100D));
		
		this.cUSE.setSelected(settings.useUniqueStartEndTasks);
		this.cAL2.setSelected(settings.preferAndToL2l);
		this.cPL2.setSelected(settings.preventL2lWithL1l);
	}
	
	public boolean isMineSwimLanes() {
		return mineSwimLanes.isSelected();
	}
	
	public boolean isReportFitness() {
		return reportFitness.isSelected();
	}
	
	public boolean isFilterLog() {
		return filterLog.isSelected();
	}
	
	public MinerSettings getSettings(){ 
		this.settings.dependencyThreshold = (this.tD.getValue() / 100D);
		this.settings.l1lThreshold = (this.tD.getValue() / 100D);
		this.settings.l2lThreshold = (this.tD.getValue() / 100D);
		this.settings.longDistanceThreshold = 0.90D;
		this.settings.patternThreshold = (this.tPG.getValue() / 100D);
		this.settings.dependencyDivisor = 1;
		this.settings.duplicateThreshold = 0.10D;
		
		this.settings.useAllConnectedHeuristics = true;
		this.settings.backwardContextSize = 0;
		this.settings.forwardContextSize = 0;
		this.settings.useLongDistanceDependency = false;
		this.settings.useUniqueStartEndTasks = this.cUSE.isSelected();
		this.settings.collapseL1l =  true;
		this.settings.preferAndToL2l = this.cAL2.isSelected();
		this.settings.preventL2lWithL1l = this.cPL2.isSelected();
		
		this.settings.classifier = XLogInfoImpl.STANDARD_CLASSIFIER;

		return this.settings; 
	}

	protected void updateFields() {
		
	}
}
