package org.processmining.plugins.bpmnminer.ui;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.processmining.plugins.bpmnminer.types.MinerSettings;
import org.processmining.plugins.kutoolbox.ui.FancyIntegerSlider;

public class FullParameterPanel extends AbstractParameterPanel {
	private static final long serialVersionUID = -1587415938421148771L;

	private FancyIntegerSlider tD, tL1, tL2, tLD, dD, tPG, dDup; //, dTas;
	private JCheckBox cAC, cDT, cLD, cUSE, cCL1, cAL2, cPL2;
	private JComboBox<?> clC;

	public FullParameterPanel(){
		this(new MinerSettings());
	}
	
	public FullParameterPanel(MinerSettings settings){
		super(settings);
	}
	
	protected void init(){
		this.addHeading("Classifier", 1);

		Object[] classifiers = new Object[] {
				XLogInfoImpl.STANDARD_CLASSIFIER,
				XLogInfoImpl.LIFECYCLE_TRANSITION_CLASSIFIER,
				XLogInfoImpl.NAME_CLASSIFIER,
				XLogInfoImpl.RESOURCE_CLASSIFIER
				};
		clC = this.addCombobox(0, classifiers, 2, false);
	
		this.addHeading("Thresholds", 3);

		this.addLabel("Dependency threshold:", 4);
		tD = this.addIntegerSlider(0, (int) (settings.dependencyThreshold * 100D), 100, 4);
		
		this.addLabel("Length-one-loops threshold:", 5);
		tL1 = this.addIntegerSlider(0, (int) (settings.l1lThreshold * 100D), 100, 5);
		
		this.addLabel("Length-two-loops threshold:", 6);
		tL2 = this.addIntegerSlider(0, (int) (settings.l2lThreshold * 100D), 100, 6);
		
		this.addLabel("Long distance dependency threshold:", 7);
		tLD = this.addIntegerSlider(0, (int) (settings.longDistanceThreshold * 100D), 100, 7);
		
		this.addLabel("Precision / generalisation threshold (higher = stronger):", 8);
		tPG = this.addIntegerSlider(-100, (int) (settings.patternThreshold * 100D), 100, 8);
		
		this.addLabel("Dependency divisor:", 9);
		dD = this.addIntegerSlider(1, settings.dependencyDivisor, 10, 9);
		
	//	this.addLabel("Task frequency threshold:", 10);
	//	dTas = this.addIntegerSlider(0, (int) (settings.taskThreshold * 100D), 100, 10);
		
		this.addLabel("Duplicate task threshold:", 10);
		dDup = this.addIntegerSlider(0, (int) (settings.duplicateThreshold * 100D), 100, 10);
		
		this.addHeading("Options", 12);

		cAC = this.addCheckbox("All-connected activities", settings.useAllConnectedHeuristics, 13, false);
		cDT = this.addCheckbox("Mine duplicate activities", 
				(settings.backwardContextSize > 0 || settings.forwardContextSize > 0), 14, false);
		cLD = this.addCheckbox("Mine long-distance dependencies", settings.useLongDistanceDependency, 15, false);
		cUSE = this.addCheckbox("Use artificial start/end activities", settings.useUniqueStartEndTasks, 16, false);
		cCL1 = this.addCheckbox("Collapse length one loops", settings.collapseL1l, 17, false);
		cAL2 = this.addCheckbox("Resolve binary conflicts where possible", settings.preferAndToL2l, 18, false);
		cPL2 = this.addCheckbox("Prevent length two loops between repeating activities", settings.preventL2lWithL1l, 19, false);
		this.updateFields();		
	}
	
	public void applySettings(MinerSettings settings){
		this.tD.setValue((int) (settings.dependencyThreshold * 100D));
		this.tL1.setValue((int) (settings.l1lThreshold * 100D));
		this.tL2.setValue((int) (settings.l2lThreshold * 100D));
		this.tLD.setValue((int) (settings.longDistanceThreshold * 100D));
		this.tPG.setValue((int) (settings.patternThreshold * 100D));
		this.dD.setValue(settings.dependencyDivisor);
	//	this.dTas.setValue((int) (settings.taskThreshold * 100D));
		this.dDup.setValue((int) (settings.duplicateThreshold * 100D));
		
		this.cAC.setSelected(settings.useAllConnectedHeuristics);
		this.cDT.setSelected(settings.backwardContextSize > 0 || settings.forwardContextSize > 0);
		this.cLD.setSelected(settings.useLongDistanceDependency);
		this.cUSE.setSelected(settings.useUniqueStartEndTasks);
		this.cCL1.setSelected(settings.collapseL1l);
		this.cAL2.setSelected(settings.preferAndToL2l);
		this.cPL2.setSelected(settings.preventL2lWithL1l);
		
		this.clC.setSelectedItem(settings.classifier);
	}
	
	public MinerSettings getSettings(){ 
		this.settings.dependencyThreshold = (this.tD.getValue() / 100D);
		this.settings.l1lThreshold = (this.tL1.getValue() / 100D);
		this.settings.l2lThreshold = (this.tL2.getValue() / 100D);
		this.settings.longDistanceThreshold = (this.tLD.getValue() / 100D);
		this.settings.patternThreshold = (this.tPG.getValue() / 100D);
		this.settings.dependencyDivisor = this.dD.getValue();
	//	this.settings.taskThreshold = (this.dTas.getValue() / 100D);
		this.settings.duplicateThreshold = (this.dDup.getValue() / 100D);
		
		this.settings.useAllConnectedHeuristics = this.cAC.isSelected();
		this.settings.backwardContextSize = this.cDT.isSelected() ? 1 : 0;
		this.settings.forwardContextSize = this.cDT.isSelected() ? 1 : 0;
		this.settings.useLongDistanceDependency = this.cLD.isSelected();
		this.settings.useUniqueStartEndTasks = this.cUSE.isSelected();
		this.settings.collapseL1l = this.cCL1.isSelected();
		this.settings.preferAndToL2l = this.cAL2.isSelected();
		this.settings.preventL2lWithL1l = this.cPL2.isSelected();
		
		this.settings.classifier = (XEventClassifier) this.clC.getSelectedItem();

		return this.settings; 
	}

	protected void updateFields() {
		
	}
}
