package org.processmining.plugins.bpmnminer.ui;

import javax.swing.JCheckBox;

import org.processmining.plugins.bpmnminer.types.MinerSettings;
import org.processmining.plugins.kutoolbox.ui.FancyIntegerSlider;

public class SimpleParameterPanel extends AbstractParameterPanel {
	private static final long serialVersionUID = -1587415938421148771L;

	private FancyIntegerSlider tD, tPG;
	private JCheckBox cDT, cLD;

	public SimpleParameterPanel(){
		this(new MinerSettings());
	}
	
	public SimpleParameterPanel(MinerSettings settings){
		super(settings);
	}
	
	protected void init(){
		this.addHeading("Thresholds", 1);

		this.addLabel("Dependency threshold:", 2);
		tD = this.addIntegerSlider(0, (int) (settings.dependencyThreshold * 100D), 100, 2);
		
		this.addLabel("Precision / generalisation threshold (higher = stronger):", 3);
		tPG = this.addIntegerSlider(-100, (int) (settings.patternThreshold * 100D), 100, 3);
		
		this.addHeading("Options", 5);

		cDT = this.addCheckbox("Mine duplicate activities", 
				(settings.backwardContextSize > 0 || settings.forwardContextSize > 0), 12, true);
		cLD = this.addCheckbox("Mine long-distance dependencies", settings.useLongDistanceDependency, 13, true);
		
		this.updateFields();		
	}
	
	public void applySettings(MinerSettings settings){
		this.tD.setValue((int) (settings.dependencyThreshold * 100D));
		this.tPG.setValue((int) (settings.patternThreshold * 100D));
		this.cDT.setSelected(settings.backwardContextSize > 0 || settings.forwardContextSize > 0);
		this.cLD.setSelected(settings.useLongDistanceDependency);
	}
	
	public MinerSettings getSettings(){ 
		this.settings.dependencyThreshold = (this.tD.getValue() / 100D);
		this.settings.l1lThreshold = (this.tD.getValue() / 100D);
		this.settings.l2lThreshold = (this.tD.getValue() / 100D);
		this.settings.longDistanceThreshold = 0.90D;
		this.settings.patternThreshold = (this.tPG.getValue() / 100D);
		this.settings.dependencyDivisor = 1;
		this.settings.duplicateThreshold = 0.10D;
		
		
		this.settings.useAllConnectedHeuristics = true;
		this.settings.backwardContextSize = this.cDT.isSelected() ? 1 : 0;
		this.settings.forwardContextSize = this.cDT.isSelected() ? 1 : 0;
		this.settings.useLongDistanceDependency = this.cLD.isSelected();
		this.settings.useUniqueStartEndTasks = false;
		this.settings.collapseL1l = true;
		this.settings.preferAndToL2l = false;

		return this.settings; 
	}

	protected void updateFields() {
		
	}
}
