package org.processmining.plugins.bpmnminer.visualization;

import javax.swing.JComponent;
import javax.swing.SwingConstants;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.jgraph.ProMGraphModel;
import org.processmining.models.jgraph.ProMJGraph;
import org.processmining.plugins.bpmnminer.causalnet.CausalGraph;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.converter.CausalNetToCausalGraph;
import org.processmining.plugins.kutoolbox.visualizators.ExportPanel;
import org.processmining.plugins.kutoolbox.visualizators.PIPPanel;
import org.processmining.plugins.kutoolbox.visualizators.ZoomPanel;

import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.hierarchical.JGraphHierarchicalLayout;

@Plugin(name = "Visualize Causal Net with Split / Join Semantics",
		parameterLabels = { "Causal Net" }, 
		returnLabels = { "CG Visualization" },
		returnTypes = { JComponent.class })

@Visualizer
public class CausalGraphVisualization {

	@PluginVariant(requiredParameterLabels = { 0 })
	public static JComponent visualize(PluginContext context, CausalNet net) {
		ProMJGraph graph = buildJGraph(net);
		CausalGraphView view = new CausalGraphView(graph);
		
		PIPPanel pip = new PIPPanel(view);
		view.addViewPanel(pip, "PIP", "View", true);
		
		ZoomPanel zoom = new ZoomPanel(view, DependencyGraphView.MAX_ZOOM);
		zoom.setHeight(200);
		view.addViewPanel(zoom, "Zoom", "View", true);
		
		ExportPanel export = new ExportPanel(view);
		view.addViewPanel(export, "Options", "Info", false);
		
		view.addInteractionViewports(pip, zoom);
				
		return view;
	}
	
	private static ProMJGraph buildJGraph(CausalNet causalNet){
		
		CausalGraph graph = CausalNetToCausalGraph.toCausalGraph(causalNet);
		
		ViewSpecificAttributeMap map = new ViewSpecificAttributeMap();
		GraphLayoutConnection layoutConnection = new GraphLayoutConnection(graph);
		
		ProMGraphModel model = new ProMGraphModel(graph);
		ProMJGraph jGraph = new ProMJGraph(model, map, layoutConnection);
		
		JGraphHierarchicalLayout layout = new JGraphHierarchicalLayout();
		layout.setDeterministic(false);
		layout.setCompactLayout(false);
		layout.setFineTuning(true);
		layout.setParallelEdgeSpacing(15);
		layout.setFixRoots(false);

		layout.setOrientation(map.get(graph, AttributeMap.PREF_ORIENTATION, SwingConstants.SOUTH));

		if(!layoutConnection.isLayedOut()){
		
			JGraphFacade facade = new JGraphFacade(jGraph);
	
			facade.setOrdered(false);
			facade.setEdgePromotion(true);
			facade.setIgnoresCellsInGroups(false);
			facade.setIgnoresHiddenCells(false);
			facade.setIgnoresUnconnectedCells(false);
			facade.setDirected(true);
			facade.resetControlPoints();
			facade.run(layout, true);
	
			java.util.Map<?, ?> nested = facade.createNestedMap(true, true);
	
			jGraph.getGraphLayoutCache().edit(nested);
			layoutConnection.setLayedOut(true);
		}
		
		jGraph.setUpdateLayout(layout);
		
		layoutConnection.updated();
		
		return jGraph;
	}
}

