package org.processmining.plugins.bpmnminer.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.jgraph.ProMJGraph;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.converter.DependencyNetToCausalNet;
import org.processmining.plugins.bpmnminer.dependencygraph.DependencyGraph;
import org.processmining.plugins.bpmnminer.dependencygraph.DependencyGraphEdge;
import org.processmining.plugins.bpmnminer.dependencygraph.DependencyGraphNode;
import org.processmining.plugins.bpmnminer.dependencygraph.DependencyNet;
import org.processmining.plugins.bpmnminer.fitness.AbstractFitness;
import org.processmining.plugins.bpmnminer.fitness.FlowFitness;
import org.processmining.plugins.bpmnminer.fitness.FlowPMFitness;
import org.processmining.plugins.bpmnminer.fitness.FuzzyFitness;
import org.processmining.plugins.bpmnminer.fitness.FuzzyPMFitness;
import org.processmining.plugins.bpmnminer.types.EventLogTaskMapper;
import org.processmining.plugins.bpmnminer.types.IntegerEventLog;
import org.processmining.plugins.bpmnminer.types.RelationalMetrics;
import org.processmining.plugins.kutoolbox.groupedlog.GroupedXLog;
import org.processmining.plugins.kutoolbox.visualizators.AbstractGraphView;

import com.fluxicon.slickerbox.ui.SlickerSliderUI;

public class FlowAnalysisView extends AbstractGraphView {

	private static final long serialVersionUID = 15797117234781L;

	private PluginContext context;

	private RelationalMetrics metrics;
	private DependencyNet dependencyNet;
	private EventLogTaskMapper mapper;
	private DependencyNet allConnectedNet, freqPathNet;
	private XLog log;
	private GroupedXLog gLog;

	private JSlider upperThresholdSlider;
	private JSlider lowerThresholdSlider;
	private JSlider frequencySlider;

	private JLabel upperThresholdLabel;
	private JLabel lowerThresholdLabel;
	private JLabel frequencyLabel;

	private JLabel flowFitnessLabel;
	private JLabel pmFitnessLabel;
	private JLabel fuzzyFitnessLabel;
	private JLabel fuzzyPmFitnessLabel;
	
	private int maxTotalInputCount = 0;
	private int maxTotalOutputCount = 0;
	@SuppressWarnings("unused")
	private int totalInputCount = 0;
	@SuppressWarnings("unused")
	private int totalOutputCount = 0;
	private Map<Integer, Integer> taskInputCount = new HashMap<Integer, Integer>();
	private Map<Integer, Integer> taskOutputCount = new HashMap<Integer, Integer>();
	private Map<Integer, Integer> maxTaskInputCount = new HashMap<Integer, Integer>();
	private Map<Integer, Integer> maxTaskOutputCount = new HashMap<Integer, Integer>();
	private float maxSize = 8;
	private float minSize = 2;
	private int maxColor = 200;
	private int minColor = 30;

	public FlowAnalysisView(PluginContext c, ScalableComponent scalableComponent, DependencyNet dn,
			RelationalMetrics m, EventLogTaskMapper map, XLog l) {
		super(scalableComponent);
		metrics = m;
		dependencyNet = dn;
		context = c;
		mapper = map;
		log = l;
		gLog = new GroupedXLog(log);

		allConnectedNet = getViewedDependencyNet(1.0D, true);
		freqPathNet = getFrequentPathsDependencyNet(1);

		initializeGui();
		setupGraph();
		adjustGraph();
	}

	private void initializeGui() {
		JPanel sliderPanel = new JPanel();
		sliderPanel.setOpaque(false);
		sliderPanel.setLayout(new BoxLayout(sliderPanel, BoxLayout.X_AXIS));

		JLabel upperHeader = new JLabel("Upper T.");
		JLabel lowerHeader = new JLabel("Lower T.");
		JLabel freqHeader = new JLabel("Freq. PIs");
		upperHeader.setSize(new Dimension(100, 25));
		upperHeader.setAlignmentX(Component.CENTER_ALIGNMENT);
		upperHeader.setForeground(Color.WHITE);
		lowerHeader.setSize(new Dimension(100, 25));
		lowerHeader.setAlignmentX(Component.CENTER_ALIGNMENT);
		lowerHeader.setForeground(Color.WHITE);
		freqHeader.setSize(new Dimension(100, 25));
		freqHeader.setAlignmentX(Component.CENTER_ALIGNMENT);
		freqHeader.setForeground(Color.WHITE);

		upperThresholdSlider = new JSlider(JSlider.VERTICAL, 0, 1000, 900);
		upperThresholdSlider.setUI(new SlickerSliderUI(upperThresholdSlider));
		upperThresholdSlider.setOpaque(false);
		upperThresholdSlider.addChangeListener(this);
		upperThresholdSlider.setAlignmentX(Component.CENTER_ALIGNMENT);

		lowerThresholdSlider = new JSlider(JSlider.VERTICAL, 0, 1000, 0);
		lowerThresholdSlider.setUI(new SlickerSliderUI(lowerThresholdSlider));
		lowerThresholdSlider.setOpaque(false);
		lowerThresholdSlider.addChangeListener(this);
		lowerThresholdSlider.setAlignmentX(Component.CENTER_ALIGNMENT);

		frequencySlider = new JSlider(JSlider.VERTICAL, 0, dependencyNet.getLog().getRows().size(), 1);
		frequencySlider.setUI(new SlickerSliderUI(frequencySlider));
		frequencySlider.setOpaque(false);
		frequencySlider.addChangeListener(this);
		frequencySlider.setAlignmentX(Component.CENTER_ALIGNMENT);

		upperThresholdLabel = new JLabel("0.900");
		upperThresholdLabel.setSize(new Dimension(100, 25));
		upperThresholdLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		upperThresholdLabel.setForeground(Color.WHITE);

		lowerThresholdLabel = new JLabel("0.000");
		lowerThresholdLabel.setSize(new Dimension(100, 25));
		lowerThresholdLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		lowerThresholdLabel.setForeground(Color.WHITE);

		frequencyLabel = new JLabel("1");
		frequencyLabel.setSize(new Dimension(100, 25));
		frequencyLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		frequencyLabel.setForeground(Color.WHITE);

		JButton blackLogsToProM = new JButton("Export Fitting");
		JButton redLogsToProM = new JButton("Export Exceptional");
		JButton greenLogsToProM = new JButton("Export Frequent");
		blackLogsToProM.setAlignmentX(Component.CENTER_ALIGNMENT);
		redLogsToProM.setAlignmentX(Component.CENTER_ALIGNMENT);
		greenLogsToProM.setAlignmentX(Component.CENTER_ALIGNMENT);
		blackLogsToProM.addActionListener(new BlackLogsExportAction());
		redLogsToProM.addActionListener(new RedLogsExportAction());
		greenLogsToProM.addActionListener(new GreenLogsExportAction());
		
		flowFitnessLabel = new JLabel("---");
		flowFitnessLabel.setSize(new Dimension(100, 25));
		flowFitnessLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		flowFitnessLabel.setForeground(Color.WHITE);

		pmFitnessLabel = new JLabel("---");
		pmFitnessLabel.setSize(new Dimension(100, 25));
		pmFitnessLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		pmFitnessLabel.setForeground(Color.WHITE);

		fuzzyFitnessLabel = new JLabel("---");
		fuzzyFitnessLabel.setSize(new Dimension(100, 25));
		fuzzyFitnessLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		fuzzyFitnessLabel.setForeground(Color.WHITE);

		fuzzyPmFitnessLabel = new JLabel("---");
		fuzzyPmFitnessLabel.setSize(new Dimension(100, 25));
		fuzzyPmFitnessLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		fuzzyPmFitnessLabel.setForeground(Color.WHITE);

		JPanel leftSliderPanel = new JPanel();
		leftSliderPanel.setOpaque(false);
		leftSliderPanel.setLayout(new BoxLayout(leftSliderPanel, BoxLayout.Y_AXIS));

		JPanel rightSliderPanel = new JPanel();
		rightSliderPanel.setOpaque(false);
		rightSliderPanel.setLayout(new BoxLayout(rightSliderPanel, BoxLayout.Y_AXIS));

		JPanel freqSliderPanel = new JPanel();
		freqSliderPanel.setOpaque(false);
		freqSliderPanel.setLayout(new BoxLayout(freqSliderPanel, BoxLayout.Y_AXIS));

		leftSliderPanel.add(upperHeader);
		leftSliderPanel.add(upperThresholdSlider);
		leftSliderPanel.add(upperThresholdLabel);
		leftSliderPanel.add(blackLogsToProM);
		leftSliderPanel.add(flowFitnessLabel);
		leftSliderPanel.add(pmFitnessLabel);
		

		rightSliderPanel.add(lowerHeader);
		rightSliderPanel.add(lowerThresholdSlider);
		rightSliderPanel.add(lowerThresholdLabel);
		rightSliderPanel.add(redLogsToProM);
		rightSliderPanel.add(fuzzyFitnessLabel);
		rightSliderPanel.add(fuzzyPmFitnessLabel);
		
		freqSliderPanel.add(freqHeader);
		freqSliderPanel.add(frequencySlider);
		freqSliderPanel.add(frequencyLabel);
		freqSliderPanel.add(greenLogsToProM);

		sliderPanel.add(leftSliderPanel);
		sliderPanel.add(rightSliderPanel);
		sliderPanel.add(freqSliderPanel);

		this.add(sliderPanel, BorderLayout.LINE_END);
		this.revalidate();
		this.repaint();

	}

	public void stateChanged(ChangeEvent evt) {
		Object source = evt.getSource();
		if (source == upperThresholdSlider) {
			updateUpperThreshold();
		} else if (source == lowerThresholdSlider) {
			updateLowerThreshold();
		} else if (source == frequencySlider) {
			updateFrequencySlider();
		} else if (source instanceof JSlider) {
			scalable.setScale(((JSlider) source).getValue() / 100.0);
			getComponent().repaint();
		}
	}

	private int getFrequencyValue() {
		return frequencySlider.getValue();
	}

	private void updateFrequencySlider() {
		int value = getFrequencyValue();
		frequencyLabel.setText(Integer.toString(value));
		freqPathNet = getFrequentPathsDependencyNet(value);
		adjustGraph();
	}

	protected double getUpperThreshold() {
		return upperThresholdSlider.getValue() / 1000.0;
	}

	protected void updateUpperThreshold() {
		double value = getUpperThreshold();
		upperThresholdLabel.setText(Double.toString(value));
		adjustGraph();
		if (upperThresholdSlider.getValueIsAdjusting() == false) {
			adjustFitness();
		}
	}

	protected double getLowerThreshold() {
		return lowerThresholdSlider.getValue() / 1000.0;
	}

	protected void updateLowerThreshold() {
		double value = getLowerThreshold();
		lowerThresholdLabel.setText(Double.toString(value));
		adjustGraph();
		if (lowerThresholdSlider.getValueIsAdjusting() == false) {
			adjustFitness();
		}
	}

	private void setupGraph() {
		// Set the labels, calculate totality values
		IntegerEventLog ieLog = dependencyNet.getLog();

		for (int t : dependencyNet.getTasks()) {
			// Set label
			getGraphNode(t).setLabel(
					"<center>" + ieLog.getLabel(t) + "<br />\r\n" + metrics.getBasicRelations().getAMeasure(t)
							+ "</center>");

			// Calculate totality values
			taskInputCount.put(t, 0);
			taskOutputCount.put(t, 0);
			maxTaskInputCount.put(t, 0);
			maxTaskOutputCount.put(t, 0);
			for (int s : dependencyNet.getTaskInputs(t)) {
				int addIn = metrics.getBasicRelations().getABMeasure(s, t);
				taskInputCount.put(t, taskInputCount.get(t) + addIn);
				if (addIn > maxTaskInputCount.get(t))
					maxTaskInputCount.put(t, addIn);
				if (addIn > maxTotalInputCount)
					maxTotalInputCount = addIn;
				totalInputCount += addIn;
			}
			for (int u : dependencyNet.getTaskOutputs(t)) {
				int addOut = metrics.getBasicRelations().getABMeasure(t, u);
				taskOutputCount.put(t, taskOutputCount.get(t) + addOut);
				if (addOut > maxTaskOutputCount.get(t))
					maxTaskOutputCount.put(t, addOut);
				if (addOut > maxTotalOutputCount)
					maxTotalOutputCount = addOut;
				totalOutputCount += addOut;

				// Set edge label
				DependencyGraphEdge<? extends DependencyGraphNode, ? extends DependencyGraphNode> arc = this
						.getGraphEdge(t, u);
				arc.getAttributeMap().put(AttributeMap.LABEL, "" + metrics.getBasicRelations().getABMeasure(t, u));
			}
		}

		this.redraw();
	}

	private void adjustGraph() {
		// Set size and color for upper threshold
		for (int t : dependencyNet.getTasks()) {
			for (int u : dependencyNet.getTaskOutputs(t)) {
				if (metrics.getBasicRelations().getABMeasure(t, u) == 0)
					continue;

				float size = minSize
						+ ((float) metrics.getBasicRelations().getABMeasure(t, u) / (float) maxTotalOutputCount)
						* (maxSize - minSize);
				int color = (int) (maxColor - ((float) metrics.getBasicRelations().getABMeasure(t, u) / (float) maxTaskOutputCount
						.get(t)) * (maxColor - minColor));

				DependencyGraphEdge<? extends DependencyGraphNode, ? extends DependencyGraphNode> arc = this
						.getGraphEdge(t, u);

				if (freqPathNet.isArc(t, u)) {
					arc.getAttributeMap().put(AttributeMap.LINEWIDTH, size);
					arc.getAttributeMap().put(AttributeMap.EDGECOLOR, new Color(10, 115, 20));
					arc.getAttributeMap().put(AttributeMap.SHOWLABEL, true);
				} else if ((allConnectedNet.isArc(t, u)) || (getConfidenceValue(t, u) >= getUpperThreshold())) {
					arc.getAttributeMap().put(AttributeMap.LINEWIDTH, size);
					arc.getAttributeMap().put(AttributeMap.EDGECOLOR, new Color(color, color, color));
					arc.getAttributeMap().put(AttributeMap.SHOWLABEL, true);
				} else if (getConfidenceValue(t, u) <= getLowerThreshold()) {
					arc.getAttributeMap().put(AttributeMap.LINEWIDTH, size);
					arc.getAttributeMap().put(AttributeMap.EDGECOLOR, Color.RED);
					arc.getAttributeMap().put(AttributeMap.SHOWLABEL, true);
				} else {
					arc.getAttributeMap().put(AttributeMap.LINEWIDTH, 0.001F);
					arc.getAttributeMap().put(AttributeMap.EDGECOLOR, Color.WHITE);
					arc.getAttributeMap().put(AttributeMap.SHOWLABEL, false);
				}
			}
		}

		this.redraw();

	}

	private void adjustFitness() {
		DependencyNet realNet = getViewedDependencyNet(true);
		CausalNet realC = DependencyNetToCausalNet.toCausalNet(realNet);
	
		AbstractFitness pmFitness = new FlowPMFitness(mapper, realC);
		AbstractFitness flowFitness = new FlowFitness(mapper, realC);
		AbstractFitness fuzzyFitness = new FuzzyFitness(mapper, realC);
		AbstractFitness fuzzyPmFitness = new FuzzyPMFitness(mapper, realC);
		pmFitness.replayLog(gLog);
		flowFitness.replayLog(gLog);
		fuzzyFitness.replayLog(gLog);
		fuzzyPmFitness.replayLog(gLog);
	
		flowFitnessLabel.setText("flow: " + Double.toString(Math.round(flowFitness.getFitness() * 100D) / 100D));
		pmFitnessLabel.setText("pm: " + Double.toString(Math.round(pmFitness.getFitness() * 100D) / 100D));
		
		fuzzyFitnessLabel.setText("fuzzy: " + Double.toString(Math.round(fuzzyFitness.getFitness() * 100D) / 100D));
		fuzzyPmFitnessLabel.setText("pm: " + Double.toString(Math.round(fuzzyPmFitness.getFitness() * 100D) / 100D));
	}

	private double getConfidenceValue(int task1, int task2) {
		double aa = (task1 == task2) ? metrics.getL1lMeasure(task1) : 0D;
		double aba = metrics.getL2lMeasure(task1, task2);
		double ab = metrics.getDependencyMeasure(task1, task2);
		return Math.max(ab, Math.max(aa, aba));
	}

	private DependencyGraphNode getGraphNode(int task) {
		ProMJGraph jgraph = (ProMJGraph) this.scalable;
		DependencyGraph graph = (DependencyGraph) jgraph.getModel().getGraph();
		return graph.getNodeByCode(task);
	}

	private DependencyGraphEdge<? extends DependencyGraphNode, ? extends DependencyGraphNode> getGraphEdge(int task1,
			int task2) {
		ProMJGraph jgraph = (ProMJGraph) this.scalable;
		DependencyGraph graph = (DependencyGraph) jgraph.getModel().getGraph();
		for (DependencyGraphEdge<? extends DependencyGraphNode, ? extends DependencyGraphNode> e : graph.getEdges()) {
			DependencyGraphNode s = e.getSource();
			DependencyGraphNode t = e.getTarget();
			if (s.getCode() == task1 && t.getCode() == task2)
				return e;
		}
		return null;
	}

	private DependencyNet getViewedDependencyNet(boolean allConnected) {
		return getViewedDependencyNet(getUpperThreshold(), allConnected);
	}

	private DependencyNet getViewedDependencyNet(double threshold, boolean allConnected) {
		DependencyNet newNet = new DependencyNet(dependencyNet.getLog());
		for (int t : dependencyNet.getTasks())
			newNet.addTask(t);

		for (int t : dependencyNet.getTasks()) {
			for (int u : dependencyNet.getTaskOutputs(t)) {
				if (metrics.getBasicRelations().getABMeasure(t, u) == 0)
					continue;

				if (getConfidenceValue(t, u) >= threshold)
					newNet.setArc(t, u);
			}
		}

		newNet.setStartTask(dependencyNet.getStartTask());
		newNet.setEndTask(dependencyNet.getEndTask());

		if (allConnected) {
			boolean faults = true;
			while (faults) {
				Set<Integer> unconnectedFromStart = newNet.getUnconnectedTasksFromStart();
				Set<Integer> unconnectedFromEnd = newNet.getUnconnectedTasksFromEnd();
				faults = !unconnectedFromStart.isEmpty() || !unconnectedFromEnd.isEmpty();
				if (!faults)
					break;
				acceptNextBestDependency(newNet, unconnectedFromStart, unconnectedFromEnd);
			}
		}

		return newNet;
	}

	private DependencyNet getFrequentPathsDependencyNet(int value) {
		DependencyNet newNet = new DependencyNet(dependencyNet.getLog());
		Set<Integer> doneRows = new HashSet<Integer>();
		int valueCounter = 0;

		for (int t : dependencyNet.getTasks())
			newNet.addTask(t);

		while (valueCounter < value) {
			int bestFreq = 0;
			int bestRow = -1;
			for (int row = 0; row < dependencyNet.getLog().getRows().size(); row++) {
				int freq = dependencyNet.getLog().getRowCount(row);
				if (freq > bestFreq && !doneRows.contains(row)) {
					bestRow = row;
					bestFreq = freq;
				}
			}
			if (bestRow == -1)
				break;

			doneRows.add(bestRow);

			int[] row = dependencyNet.getLog().getRow(bestRow);
			for (int i = 0; i < row.length - 1; i++)
				newNet.setArc(row[i], row[i + 1]);

			valueCounter++;
		}

		newNet.setStartTask(dependencyNet.getStartTask());
		newNet.setEndTask(dependencyNet.getEndTask());

		return newNet;
	}

	private void acceptNextBestDependency(DependencyNet newNet, Set<Integer> possibleTailTasks,
			Set<Integer> possibleHeadTasks) {
		double bestDependency = -1D;
		boolean isL2L = false;
		int a = -1;
		int b = -1;
		for (int c : newNet.getTasks()) {
			for (int d : possibleTailTasks) {
				double measure = metrics.getDependencyMeasure(c, d);
				double measure2 = metrics.getL2lMeasure(c, d);
				if (newNet.getStartTask() == d || newNet.getEndTask() == c || c == d || newNet.isArc(c, d)
						|| ((measure < bestDependency) && (measure2 < bestDependency)))
					continue;
				bestDependency = Math.max(measure2, measure);
				a = c;
				b = d;
				isL2L = measure2 > measure;
			}
			for (int d : possibleHeadTasks) {
				double measure = metrics.getDependencyMeasure(d, c);
				double measure2 = metrics.getL2lMeasure(d, c);
				if (newNet.getEndTask() == d || newNet.getStartTask() == c || c == d || newNet.isArc(d, c)
						|| ((measure < bestDependency) && (measure2 < bestDependency)))
					continue;
				bestDependency = Math.max(measure2, measure);
				a = d;
				b = c;
				isL2L = measure2 > measure;
			}
		}
		if (a != -1 && b != -1) {
			newNet.setArc(a, b);
			if (isL2L)
				newNet.setArc(b, a);
		}
	}

	private class BlackLogsExportAction implements ActionListener {
		final private XFactory factory = XFactoryRegistry.instance().currentDefault();
		@Override
		public void actionPerformed(ActionEvent e) {
			DependencyNet realNet = getViewedDependencyNet(true);
			CausalNet realC = DependencyNetToCausalNet.toCausalNet(realNet);
			AbstractFitness pmFitness = new FlowPMFitness(mapper, realC);
			
			String name = "Fodina Export: Fitting Flows ("+getUpperThreshold()+")";
			XLog log = factory.createLog();
			log.getAttributes().put("concept:name", 
					factory.createAttributeLiteral("concept:name", name, XConceptExtension.instance()));
			
			for (int i = 0; i < gLog.size(); i++) {
				pmFitness.reset();
				pmFitness.replayTrace(gLog.get(i).get(0), gLog.get(i).size());
				if (pmFitness.getFitness() < 1D)
					continue;
				
				log.addAll(gLog.get(i));
			}
			if (log.size() > 0)
				context.getProvidedObjectManager().createProvidedObject(
					name, log, XLog.class, context);
		}
	}
	
	private class RedLogsExportAction implements ActionListener {
		final private XFactory factory = XFactoryRegistry.instance().currentDefault();
		@Override
		public void actionPerformed(ActionEvent e) {
			String name = "Fodina Export: Low-Frequent Flows ("+getLowerThreshold()+")";
			XLog log = factory.createLog();
			log.getAttributes().put("concept:name", 
					factory.createAttributeLiteral("concept:name", name, XConceptExtension.instance()));
			
			for (int[] row : dependencyNet.getLog().getRows()) {
				boolean add = false;
				for (int p = 0; p < row.length-1; p++) {
					if (getConfidenceValue(row[p], row[p+1]) < getUpperThreshold()
							&& getConfidenceValue(row[p], row[p+1]) <= getLowerThreshold()) {
						add = true;
						break;
					}
				}
				if (!add)
					continue;
				Set<XTrace> matches = mapper.getMatchingTraces(row);
				log.addAll(matches);
			}
			if (log.size() > 0)
				context.getProvidedObjectManager().createProvidedObject(
					name, log, XLog.class, context);
		}
	}
	
	private class GreenLogsExportAction implements ActionListener {
		final private XFactory factory = XFactoryRegistry.instance().currentDefault();
		@Override
		public void actionPerformed(ActionEvent e) {
			String name = "Fodina Export: Frequent Paths ("+getFrequencyValue()+")";
			XLog log = factory.createLog();
			log.getAttributes().put("concept:name", 
					factory.createAttributeLiteral("concept:name", name, XConceptExtension.instance()));
			
			Set<Integer> doneRows = new HashSet<Integer>();
			int value = getFrequencyValue();
			int valueCounter = 0;
			while (valueCounter < value) {
				int bestFreq = 0;
				int bestRow = -1;
				for (int row = 0; row < dependencyNet.getLog().getRows().size(); row++) {
					int freq = dependencyNet.getLog().getRowCount(row);
					if (freq > bestFreq && !doneRows.contains(row)) {
						bestRow = row;
						bestFreq = freq;
					}
				}
				if (bestRow == -1)
					break;

				doneRows.add(bestRow);
				Set<XTrace> matches = mapper.getMatchingTraces(dependencyNet.getLog().getRow(bestRow));
				log.addAll(matches);
				
				valueCounter++;
			}
			if (log.size() > 0)
				context.getProvidedObjectManager().createProvidedObject(
					name, log, XLog.class, context);
		}
	}


}
