package org.processmining.plugins.bpmnminer.fitness;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.bpmnminer.causalnet.CausalNet;
import org.processmining.plugins.bpmnminer.converter.CausalNetToDependencyNet;
import org.processmining.plugins.bpmnminer.dependencygraph.DependencyNet;
import org.processmining.plugins.bpmnminer.types.EventLogTaskMapper;

public class FuzzyFitness extends BasicFitness {

	protected int badTasks;
	protected int goodTasks;
	
	protected int badTraces;
	protected int goodTraces;
	
	protected DependencyNet dependencyNet;
	
	public FuzzyFitness(EventLogTaskMapper m, CausalNet n) {
		super(m, n);
		dependencyNet = CausalNetToDependencyNet.toDependencyNet(n);
	}
	
	public void reset() {
		badTasks = 0;
		goodTasks = 0;
		badTraces = 0;
		goodTraces = 0;
	}
	
	public double getFitness() {
		return (double)goodTasks / ((double)goodTasks + (double)badTasks);
	}
	
	public void replayTrace(XTrace trace, int traceCount) {
		String[] eventrow = mapper.classifyTrace(trace);
		int[] taskrow = mapper.getIntegerTrace(trace);
		
		Set<Integer> enabledTasks = new HashSet<Integer>();
		
		int seqBad = 0;
		int seqGood = 0;

		for (int i = 0; i < taskrow.length; i++) {
			Integer currentTask = getBestTask(eventrow, taskrow, i, null);
			taskrow[i] = (currentTask == null) ? -9999 : currentTask;
			
			if (currentTask == null || 
					(!enabledTasks.contains(currentTask) && dependencyNet.getTaskInputs(currentTask).size() > 0))
				seqBad++;
			else {
				seqGood++;
				enabledTasks.remove(currentTask);
			}
			
			for (int t : dependencyNet.getTaskOutputs(currentTask))
				enabledTasks.add(t);
		}
		
		badTasks += seqBad * traceCount;
		goodTasks += seqGood * traceCount;
		
		if (seqBad > 0)
			badTraces += traceCount;
		else
			goodTraces += traceCount;
	}
}
